public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
* [PATCH] resources: fix call to alignf() in allocate_resource()
@ 2009-12-20  9:50 Dominik Brodowski
  2009-12-20 18:33 ` Linus Torvalds
  0 siblings, 1 reply; 12+ messages in thread
From: Dominik Brodowski @ 2009-12-20  9:50 UTC (permalink / raw)
  To: Linus Torvalds, Bjorn Helgaas; +Cc: Yinghai Lu, Jesse Barnes, linux-kernel

The second parameter to alignf() in allocate_resource() must
reflect what new resource is attempted to be allocated, else
functions like pcibios_align_resource() (at least on x86) or
pcmcia_align() can't work correctly.

Commit 1e5ad9679016275d422e36b12a98b0927d76f556 broke this by
setting the "new" resource until we're about to return success.
To keep the resource untouched when allocate_resource() fails,
a "tmp" resource is introduced.

CC: Linus Torvalds <torvalds@linux-foundation.org>
CC: Yinghai Lu <yhlu.kernel@gmail.com>
CC: Bjorn Helgaas <bjorn.helgaas@hp.com>
CC: Jesse Barnes <jbarnes@virtuousgeek.org>
Signed-off-by: Dominik Brodowski <linux@dominikbrodowski.net>

diff --git a/kernel/resource.c b/kernel/resource.c
index dc15686..af96c1e 100644
--- a/kernel/resource.c
+++ b/kernel/resource.c
@@ -308,37 +308,37 @@ static int find_resource(struct resource *root, struct resource *new,
 			 void *alignf_data)
 {
 	struct resource *this = root->child;
-	resource_size_t start, end;
+	struct resource tmp = *new;
 
-	start = root->start;
+	tmp.start = root->start;
 	/*
 	 * Skip past an allocated resource that starts at 0, since the assignment
 	 * of this->start - 1 to new->end below would cause an underflow.
 	 */
 	if (this && this->start == 0) {
-		start = this->end + 1;
+		tmp.start = this->end + 1;
 		this = this->sibling;
 	}
 	for(;;) {
 		if (this)
-			end = this->start - 1;
+			tmp.end = this->start - 1;
 		else
-			end = root->end;
-		if (start < min)
-			start = min;
-		if (end > max)
-			end = max;
-		start = ALIGN(start, align);
+			tmp.end = root->end;
+		if (tmp.start < min)
+			tmp.start = min;
+		if (tmp.end > max)
+			tmp.end = max;
+		tmp.start = ALIGN(tmp.start, align);
 		if (alignf)
-			alignf(alignf_data, new, size, align);
-		if (start < end && end - start >= size - 1) {
-			new->start = start;
-			new->end = start + size - 1;
+			alignf(alignf_data, &tmp, size, align);
+		if (tmp.start < tmp.end && tmp.end - tmp.start >= size - 1) {
+			new->start = tmp.start;
+			new->end = tmp.start + size - 1;
 			return 0;
 		}
 		if (!this)
 			break;
-		start = this->end + 1;
+		tmp.start = this->end + 1;
 		this = this->sibling;
 	}
 	return -EBUSY;

^ permalink raw reply related	[flat|nested] 12+ messages in thread

end of thread, other threads:[~2010-01-15 18:23 UTC | newest]

Thread overview: 12+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
@@ -308,37 +308,37 @@ static int find_resource(struct resource *root, struct resource *new,
 	/*
 	 * Skip past an allocated resource that starts at 0, since the assignment
-	 * of this->start - 1 to new->end below would cause an underflow.
+	 * of this->start - 1 to tmp->end below would cause an underflow.
 	 */
 	if (this && this->start == 0) {
2009-12-20  9:50 [PATCH] resources: fix call to alignf() in allocate_resource() Dominik Brodowski
2009-12-20 18:33 ` Linus Torvalds
2009-12-20 22:12   ` Bjorn Helgaas
2009-12-21  8:40   ` Dominik Brodowski
2009-12-21 16:03     ` Linus Torvalds
2010-01-01 16:38       ` [RFC] improve alignf() calling convention " Dominik Brodowski
2010-01-01 16:40         ` [PATCH 1/2] resource: align functions now return start of resource Dominik Brodowski
2010-01-01 22:12           ` Yinghai Lu
2010-01-03 16:30             ` Dominik Brodowski
2010-01-15 18:23           ` Jesse Barnes
2010-01-01 16:40         ` [PATCH 2/2] resource: mark struct resource as const Dominik Brodowski
2010-01-14  7:28       ` why can not receive kernel mail anymore? goldenwang1979

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox